#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "SDL.h"
#include "SDL_ttf.h"
#include "darkbattle.h"
#include "randomize.h"

#define MAX_STARS 	75   // it explayns itself... doesn'it ?
#define BUF_BULLETS	90
#define SPEED_SH  	13   // speed of ship
#define SPEED_BG  	8    // speed of background
#define SPEED_BU  	20   // speed of bullet_0_imgs
#define STATUS_BAR_H	18


extern Uint8 	alien_type;
extern Uint8 	released_down;
extern Uint8 	released_up;
extern Uint8 	released_left;
extern Uint8 	released_right;
extern Uint8 	released_fire;
extern Uint32	pause;
extern Uint8 	nAlien;
extern SDL_Surface *ship_img_a;
extern SDL_Surface *ship_img_b;
extern SDL_Surface *ship_img_c;
extern SDL_Surface *screen;
extern SDL_Surface *al_img;
extern SDL_Surface *ar_img;
extern SDL_Surface *ad_img;
extern SDL_Surface *au_img;
extern SDL_Surface *ald_img;
extern SDL_Surface *ard_img;
extern SDL_Surface *alu_img;
extern SDL_Surface *aru_img;
extern SDL_Surface *expl1_img;
extern SDL_Surface *expl2_img;
extern SDL_Surface *expl3_img;
extern SDL_Surface *bullet_0_img;
extern SDL_Surface *open_1;
extern SDL_Surface *met_a1, *met_a2, *met_a3, *met_a4, *met_a5;

Uint32  now;
Uint32  time_scheme1, time_scheme2;
Uint32	score;
Uint8 	r, g, b;
Uint8 	nLives;
Uint8 	nBullet;
Uint8 	nExpl;
Uint8 	nMetFrame;

TTF_Font 	*font;

Star 		star[MAX_STARS];
Darkship	darkship;
Bullet		bullet[BUF_BULLETS];
Explosion	explosion[BUF_EXPLOSION];
Meteor		meteor;

struct map_alien_a map_ala;
struct map_ship_a  map_shipa;
struct map_bul     map_bulA;

void init_var(void) {
	released_down   = 1;
	released_up     = 1;
	released_right  = 1;
	released_left   = 1;
	released_fire   = 1;
	nLives	        = 3;
	nBullet         = 0;
	score           = 0;
	time_scheme1    = 2000;
	time_scheme2    = 5000;
	pause           = 0;
	nAlien          = 0;
	nExpl           = 0;
}

void init_explosions(void) {
	Uint8 i;
	for(i=0; i<BUF_EXPLOSION; i++)
		explosion[i].ttl = 6;
}

void init_stars(void) {
	Uint8  i;

	for(i=0; i<MAX_STARS; i++) {
		star[i].x = randomize(0, 640);
		star[i].y = randomize(0, 480 - STATUS_BAR_H);
#ifdef DEBUG
		printf("x: %d\ty: %d\n", star[i].x, star[i].y);
#endif
		star[i].pos.x = star[i].x;
		star[i].pos.y = star[i].y;
		star[i].pos.w = 1;
		star[i].pos.h = 1;
		r = g = b = randomize(100, 255);
		star[i].color = SDL_MapRGB(screen->format, r, g, b);
		SDL_FillRect(screen, &star[i].pos, star[i].color);
	}
}

void display_img(SDL_Surface *img, SDL_Rect *pos) {
	SDL_BlitSurface(img, NULL, screen, pos);
}

void erase_old_stars(void) {
	Uint8 i;
	for(i=0; i<MAX_STARS; i++) {
		SDL_FillRect(screen, &star[i].pos, SDL_MapRGB(screen->format, 0, 0, 0));
	}
}

void make_new_stars(void) {
	Uint8 i;
	for(i=0; i<MAX_STARS; i++) {
		star[i].y += SPEED_BG;
		if(star[i].y > 480 - STATUS_BAR_H - 3) {
			star[i].y = 0;
			star[i].x = randomize(5, 635);
			r = g = b = randomize(100, 255);
			star[i].color = SDL_MapRGB(screen->format, r, g, b);
		}
		star[i].pos.x = star[i].x;
		star[i].pos.y = star[i].y;
		star[i].pos.w = 1;
		star[i].pos.h = 1;
		SDL_FillRect(screen, &star[i].pos, star[i].color);
	}
}

void init_statusbar(void) {
	SDL_Rect bar, bar_internal;
	bar.x = 0;
	bar.y = 480 - STATUS_BAR_H;
	bar.h = STATUS_BAR_H;
	bar.w = 680;
	bar_internal.x = 2;
	bar_internal.y = 480 - STATUS_BAR_H + 1;
	bar_internal.h = STATUS_BAR_H - 3;
	bar_internal.w = 680 - 5;

	SDL_FillRect(screen, &bar, SDL_MapRGB(screen->format, 200, 0, 0));
	SDL_FillRect(screen, &bar_internal, SDL_MapRGB(screen->format, 0, 0, 0));

}

void update_bar(void) {
	SDL_Surface 	*score_ttf;
	SDL_Rect	score_rect, weap;
	SDL_Color	color_score;
	char		score_str[256];

	init_statusbar();

	memset(score_str, 0, 256);
	sprintf(score_str, "Score: %-5.1d    Lives: %-3.1d    Weapon: ", score, nLives);
	color_score.r = 255;
	color_score.g = 0;
	color_score.b = 150;
#ifdef DEBUG
	printf("%s\n", score_str);
#endif
	score_ttf = TTF_RenderText_Blended(font, score_str, color_score);
	score_rect.x = 50;
	score_rect.y = 480 - STATUS_BAR_H + 1;
	score_rect.w = score_ttf->w;
	score_rect.h = score_ttf->h;
	weap.x = score_rect.x + score_ttf->w + 5;
	weap.y = score_rect.y + 3;
	if(darkship.weapon == 0) {
		weap.w = bullet_0_img->w;
		weap.h = bullet_0_img->h;
		display_img(bullet_0_img, &weap);
	}
	display_img(score_ttf, &score_rect);
	SDL_FreeSurface(score_ttf);
}

void make_boom(int x, int y) {
	explosion[nExpl].img = expl3_img;
	explosion[nExpl].x = x;
	explosion[nExpl].y = y;
	explosion[nExpl].pos.x = explosion[nExpl].x;
	explosion[nExpl].pos.y = explosion[nExpl].y;
	explosion[nExpl].pos.w = SPRITE_SIZE;
	explosion[nExpl].pos.h = SPRITE_SIZE;
	nExpl++;
	nExpl > BUF_EXPLOSION ? nExpl = 0 : nExpl;
}

void check_boom(void) {
	Uint8 i;

	for(i=0; i<BUF_EXPLOSION; i++) {
		if(explosion[i].img != NULL) {
			if(explosion[i].ttl == 0) {
					explosion[i].img = NULL;
					explosion[i].ttl = 6;
			}
			if(explosion[i].ttl == 1 || explosion[i].ttl == 4)
					display_img(expl1_img, &explosion[i].pos);
			if(explosion[i].ttl == 2 || explosion[i].ttl == 5)
					display_img(expl2_img, &explosion[i].pos);
			if(explosion[i].ttl == 3 || explosion[i].ttl == 6)
					display_img(expl3_img, &explosion[i].pos);
			explosion[i].ttl--;
		}
	}
}

int check_maps(SDL_Surface *frm, SDL_Surface *dst, int xa, int ya, int xb, int yb, int code) {
	int offset_x, offset_y;
	int frm_jump_at_x, frm_jump_at_y, dst_jump_at_y, dst_jump_at_x;
	int i, j;

	Uint8 map_frm[SPRITE_SIZE*SPRITE_SIZE];
	Uint8 map_dst[SPRITE_SIZE*SPRITE_SIZE];
	Uint8 map_bul[frm->w * frm->h];

	frm == bullet_0_img ? memcpy(map_frm, map_bulA.A, BUL_H*BUL_W):NULL;

	frm == ship_img_a ? memcpy(map_frm, map_shipa.map_a, SPRITE_SIZE*SPRITE_SIZE):NULL;
	frm == ship_img_b ? memcpy(map_frm, map_shipa.map_b, SPRITE_SIZE*SPRITE_SIZE):NULL;
	frm == ship_img_c ? memcpy(map_frm, map_shipa.map_c, SPRITE_SIZE*SPRITE_SIZE):NULL;

	dst == al_img ? memcpy(map_dst, map_ala.map_l, SPRITE_SIZE*SPRITE_SIZE):NULL;
	dst == ar_img ? memcpy(map_dst, map_ala.map_r, SPRITE_SIZE*SPRITE_SIZE):NULL;
	dst == au_img ? memcpy(map_dst, map_ala.map_u, SPRITE_SIZE*SPRITE_SIZE):NULL;
	dst == ad_img ? memcpy(map_dst, map_ala.map_d, SPRITE_SIZE*SPRITE_SIZE):NULL;
	dst == alu_img ? memcpy(map_dst, map_ala.map_lu, SPRITE_SIZE*SPRITE_SIZE):NULL;
	dst == aru_img ? memcpy(map_dst, map_ala.map_ru, SPRITE_SIZE*SPRITE_SIZE):NULL;
	dst == ald_img ? memcpy(map_dst, map_ala.map_ld, SPRITE_SIZE*SPRITE_SIZE):NULL;
	dst == ard_img ? memcpy(map_dst, map_ala.map_rd, SPRITE_SIZE*SPRITE_SIZE):NULL	;

	if(code == 0) {
		offset_x = (xb + dst->w) - xa;
		offset_y = (yb + dst->h) - ya;
		dst_jump_at_x = dst->w - offset_x;
		dst_jump_at_y = dst->h - offset_y;
		for(i=0; i<offset_x; i++) {
			for(j=0; j<offset_y; j++) {
				if(map_frm[i*j+i] && map_dst[(dst_jump_at_y+i)*dst_jump_at_x+dst_jump_at_x+j])
					return 1;
			}
		}
		return 0;
	}

}

int check_crash(void) {
	Uint8 i;
	for(i=0; i<BUF_ALIEN; i++) {
		//alieno sopra a sinistra codice: 0
		if( (darkship.x > alien[i].x) && (darkship.x < alien[i].x + alien[i].w)) {
			if( darkship.y < alien[i].y + alien[i].h) {
				if(darkship.y + darkship.h > alien[i].y) {
					if(check_maps(darkship.img, alien[i].img, darkship.x, darkship.y,
					alien[i].x, alien[i].y, 0)) {
						printf("boom!\n");
						make_boom(darkship.x, darkship.y);
						make_boom(alien[i].x, alien[i].y);
					}
				}
			}
		}
	}
}

void init_ship(void) {
	darkship.img = ship_img_a;
	darkship.w = darkship.img->w;
	darkship.h = darkship.img->h;
	darkship.x = (640 >> 1) - (darkship.w >> 1);
	darkship.y = 400;
	darkship.pos.x = darkship.x;
	darkship.pos.y = darkship.y;
	darkship.pos.w = darkship.w;
	darkship.pos.h = darkship.h;
	darkship.is_moving = 0;
	darkship.H_dir = 0;
	darkship.V_dir = 0;
	darkship.weapon = 0;
}

void calc_ship_pos(void) {
	darkship.x += SPEED_SH * darkship.H_dir;
	darkship.y += SPEED_SH * darkship.V_dir;
	if(darkship.x < 0) {
		darkship.x = 0;
		darkship.H_dir = 0;
	}
	if(darkship.x + darkship.img->w > 640 ) {
		darkship.x = 640 - darkship.img->w;
		darkship.H_dir = 0;
	}
	if(darkship.y < 0) {
		darkship.y = 0;
		darkship.V_dir = 0;
	}
	if(darkship.y + STATUS_BAR_H + darkship.img->h + 3 > 480 ) {
		darkship.y = 480 - darkship.img->h - STATUS_BAR_H - 3;
		darkship.V_dir = 0;
	}
	darkship.pos.x = darkship.x;
	darkship.pos.y = darkship.y;
	darkship.pos.w = darkship.img->w;
	darkship.pos.h = darkship.img->h;
	if(darkship.H_dir == 0)
		darkship.img = ship_img_a;
	if(darkship.H_dir == -1)
		darkship.img = ship_img_b;
	if(darkship.H_dir == 1)
		darkship.img = ship_img_c;
	check_crash();
	display_img(darkship.img, &darkship.pos);
}

void erase_old_ship(void) {
	SDL_FillRect(screen, &darkship.pos, SDL_MapRGB(screen->format, 0, 0, 0));
}

void do_shoot(void) {
	Uint8 i;

	if(darkship.weapon == 0)
		bullet[nBullet].img = bullet_0_img;
	bullet[nBullet].x = darkship.x + (darkship.w >> 1) -
					(bullet[nBullet].img->w >> 1);
	bullet[nBullet].y = darkship.y + bullet[nBullet].img->h;
	bullet[nBullet].w = bullet[nBullet].img->w;
	bullet[nBullet].h = bullet[nBullet].img->h;
	bullet[nBullet].pos.x = bullet[nBullet].x;
	bullet[nBullet].pos.y = bullet[nBullet].y;
	bullet[nBullet].pos.w = bullet[nBullet].w;
	bullet[nBullet].pos.h = bullet[nBullet].h;
	if(nBullet++ == BUF_BULLETS)
		nBullet = 0;
}

void calc_bullets_pos(void) {
	Uint8 i;
	for(i=0; i<BUF_BULLETS; i++) {
		if(bullet[i].img != NULL) {
			bullet[i].y -= SPEED_BU;
			bullet[i].pos.y = bullet[i].y;
			if(bullet[i].y + bullet[i].h < 0) {
				bullet[i].img = (void *)NULL;
			}
			display_img(bullet[i].img, &bullet[i].pos);
		}
	}
}

void check_bang(void) {
	Uint8 i, j;

	for(i=0; i<BUF_ALIEN; i++) {
		for(j=0; j<BUF_BULLETS; j++) {
			if(bullet[j].img != NULL) {
				if((bullet[j].x + bullet[j].w / 2 < alien[i].x + alien[i].w) &&	
				   (bullet[j].x + bullet[j].w / 2 > alien[i].x) &&
				   (bullet[j].y < alien[i].y + alien[i].h) &&
				   (bullet[j].y > alien[i].y - SPEED_BU - 1)) {
					//if(check_maps(bullet[j].img, alien[i].img,
					//bullet[j].x, bullet[j].y,
					//alien[i].x, alien[i].y, 0) == 1) {
					make_boom(alien[i].x, alien[i].y);
					alien[i].img = NULL;
					alien[i].x = NULL;
					alien[i].y = NULL;
					alien[i].w = NULL;
					alien[i].h = NULL;
					alien[i].destroyed = 1;
					score += alien[i].points;
					update_bar();
					SDL_FillRect(screen, &bullet[j].pos, SDL_MapRGB(screen->format, 0, 0, 0));
					bullet[j].img = NULL;
					bullet[j].x = NULL;
					bullet[j].y = NULL;
					bullet[j].w = NULL;
					bullet[j].h = NULL;
				}
			}
		}
	}
}

void erase_old_bullets(void) {
	Uint8 i;
	for(i=0; i<BUF_BULLETS; i++) {
		if(bullet[i].img != NULL)
			SDL_FillRect(screen, &bullet[i].pos, SDL_MapRGB(screen->format, 0, 0, 0));
	}
}

void init_meteor(void) {
	meteor.img = met_a1;
	meteor.frame = 0;
	meteor.x = randomize(20, 620);
	meteor.y = 0 - meteor.img->h;
	meteor.pos.x = meteor.x;
	meteor.pos.y = meteor.y;
	meteor.pos.w = SPRITE_SIZE;
	meteor.pos.h = SPRITE_SIZE;
}

void erase_old_meteor(void) {
	SDL_Rect r;
	r.x = meteor.x;
	r.y = meteor.y;
	r.w = SPRITE_SIZE;
	r.h = SPRITE_SIZE;
	SDL_FillRect(screen, &r, SDL_MapRGB(screen->format, 0, 0, 0));
}

void check_meteor(void) {
	if(meteor.frame == 0)
		meteor.img = met_a1;
	if(meteor.frame == 1)
		meteor.img = met_a2;
	if(meteor.frame == 2)
		meteor.img = met_a3;
	if(meteor.frame == 3)
		meteor.img = met_a4;
	if(meteor.frame == 4)
		meteor.img = met_a5;
	meteor.frame++;
	if(meteor.frame == 5)
		meteor.frame = 0;
	meteor.x += randomize(1, 5)*randomize(-2, 2);
	meteor.y += randomize(1, 5);
	meteor.pos.x = meteor.x;
	meteor.pos.y = meteor.y;
	display_img(meteor.img, &meteor.pos);
}

void do_game(void) {
	Uint8 scheme = 0;
	while(1) {
		now = SDL_GetTicks();
		if(now - pause > time_scheme1 && scheme == 0) {
			alien_type = 0;
			scheme  = 1;
			init_scheme(scheme);
		}
		if(now - pause - time_scheme1 > time_scheme2 && scheme == 1) {
			alien_type = 0;
			scheme  = 2;
			init_scheme(scheme);
		}
		wait_event();
		erase_old_stars();
		if(scheme)
			erase_old_alien();
		erase_old_ship();
		erase_old_bullets();
		erase_old_meteor();
		make_new_stars();
		calc_bullets_pos();
		calc_ship_pos();
		calc_alien_pos(scheme);
		check_bang();
		check_boom();
		//check_meteor();
		SDL_UpdateRect(screen, 0, 0, 0, 0);
		SDL_Delay(30);
	}
}

void splash_screen(void) {
	SDL_Rect r;
	SDL_Event event;
	r.x = 0;
	r.y = 0;
	r.w = 640;
	r.h = 480;
	display_img(open_1, &r);
	SDL_UpdateRect(screen, 0, 0, 0, 0);
	while(SDL_WaitEvent(&event)) {
		if((event.type == SDL_KEYDOWN) && (event.key.keysym.sym == SDLK_RETURN)) {
			SDL_FillRect(screen, &r, SDL_MapRGB(screen->format, 0, 0, 0));
			SDL_UpdateRect(screen, 0, 0, 0, 0);
			update_bar();
			do_game();
		}
	}
}

int main(int argc, char **argv) {
	SDL_Surface *icon;

	if((SDL_Init(SDL_INIT_VIDEO)==-1)) {
		printf("Could not initialize SDL: %s.\n", SDL_GetError());
		exit(-1);
	}
	screen = SDL_SetVideoMode(640, 480, 32, SDL_SWSURFACE);
	TTF_Init();
	atexit(SDL_Quit);
	if(load_images() == -1) {
		fprintf(stderr, "Error loading images !\n");
		exit(-1);
	}
	icon = SDL_LoadBMP("Images/icon.bmp");
	SDL_WM_SetCaption("  -= Dark Battle v0.2b =-  ", "dark battle");
	SDL_WM_SetIcon(icon, NULL);
	font = TTF_OpenFont("luximr.ttf", 12);
	if(font == NULL)
		fprintf(stderr, "Error loading font !\n");
	TTF_SetFontStyle(font, 0);
	init_var();
	init_stars();
	init_ship();
	init_statusbar();
	init_explosions();
	init_meteor();
	map_aliens();
	map_ships();
	splash_screen();
	//do_game();

	TTF_Quit();
	unload_images();
	SDL_FreeSurface(screen);
	exit(0);
}
